/*
 *  Copyright (C) 2006 David Kozub <zub at linux.fjfi.cvut.cz.nospam>
 *
 *  This program is free software; you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation; either version 2 of the License, or
 *  (at your option) any later version.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with this program; if not, write to the Free Software
 *  Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 */

#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <errno.h>

#include "to_utf8.h"
#include "trans_tbl.h"
#include "tbl_parse.h"

const char *progname="cp2utf8";
const char *DEFAULT_PATH="/usr/share/kbd/unimaps/";

void print_help_and_exit()
{
  printf("Usage: %s translation_table_file\n"
         "Translates standard input according to translation table into UTF8 to "
         "standard output.\n"
         "translation_table_file is a file in the same format as that used by "
         "kbd. See setfont(8).\n"
         "translation_table_file is searched for as given, if it's not found, "
         "\"%s\" is prepended to the path.\n",progname,DEFAULT_PATH);
  exit(1);
}

int main(int argc, char *argv[])
{
  trans_tbl t;
  FILE *f;
  int c;
  
  if (argc>1) progname=argv[0];
  
  if (argc!=2 || !strcmp(argv[1],"-h") || !strcmp(argv[1],"--help"))
    print_help_and_exit();
    
  f=fopen(argv[1],"r");
  if (!f && errno==ENOENT) {
    // retry with default path
    char *s=(char*)malloc(strlen(argv[1])+sizeof(DEFAULT_PATH)+100);
    
    strcpy(s,DEFAULT_PATH);
    strcat(s,argv[1]);
    f=fopen(s,"r");
    free(s);
    
    if (!f) {
      perror("cannot open translation table file (tried default path)");
      exit(1);
    }
  }
  
  if (!f) {
    perror("cannot open translation table file");
    exit(1);
  }
  
  read_table(f,t);
  
  // perform actual conversion
  while ((c=getchar())!=EOF) {
    const uni_str *s=t.getAt((unsigned)c);
    
    if (!s) {
/*      fprintf(stderr,"Warning: no character assigned to byte 0x%02x.\n",
        (unsigned)c);*/
      continue; // assume no entry = translate to empty string
    }
    
    for (unsigned i=0;i<s->length();i++) {
      printf("%s",conv_to_utf8(s->getCharAt(i)));
    }
  }

  return 0;
}
