/*
 *  Copyright (C) 2006 David Kozub <zub at linux.fjfi.cvut.cz.nospam>
 *
 *  This program is free software; you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation; either version 2 of the License, or
 *  (at your option) any later version.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with this program; if not, write to the Free Software
 *  Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 */

#ifndef __UNI_STR_H
#define __UNI_STR_H

#include <string.h>
#include <assert.h>

/*
 A "container" for storing unicode string (as unicode characters). Can
 contain character U+0000 (Well, not too much needed here, except for the
 byte 0x00, which more or less always maps to U+0000. I could have solved this
 special case in some other way, though...)
*/

class uni_str {
private:
  unsigned len;
  unsigned *data;

  void copy_from(const uni_str &s);
  
public:
  uni_str(): len(0), data(NULL) {};
  uni_str(unsigned* d, unsigned l);
  uni_str(const uni_str &s);
  ~uni_str();
  
  void set(unsigned* d, unsigned l);
  
  inline unsigned length() const
  {
    return len;
  }
  
  inline unsigned getCharAt(unsigned pos) const
  {
    assert(pos>=0);
    assert(pos<len);
    
    // I don't wanna introduce exceptions here, I want the program to be
    // compact.
    
    return data[pos];
  }
  
  uni_str& operator=(const uni_str s);
};

#endif // __UNI_STR_H
